@@ -9,7 +9,23 @@ use Carp;
 use Module::Loaded;
 use Monkey::Patch qw(:all);
 
-our $VERSION = '0.03'; # VERSION
+our $VERSION = '0.04'; # VERSION
+
+# match versions specification string, e.g. whether target '2.07' is in '1.23
+# 1.24 /^2\..+$/' (answer: it is)
+sub __match_v {
+    my ($target, $spec) = @_;
+
+    my @v = split /[,;]?\s+/, $spec;
+    for (@v) {
+        if (s!^/(.*)/$!$1!) {
+            return 1 if $target ~~ /$_/;
+        } else {
+            return 1 if $target eq $_;
+        }
+    }
+    0;
+}
 
 my %applied_patches; # key = targetmod, value = [patchmod1, ...]
 
@@ -20,6 +36,8 @@ sub import {
     my ($self, %args) = @_;
     my $handle = \%{"$self\::handle"}; #W
 
+    my $pre = $self; # error message prefix
+
     # already patched, ignore
     return if keys %$handle;
 
@@ -36,15 +54,15 @@ sub import {
 
     my $target = $self;
     $target =~ s/(?<=\w)::patch::\w+$//
-        or croak "BUG: Bad patch module name '$target', it should end with ".
-            "'::patch::something'";
+        or die "BUG: Bad patch module name '$target', it should ".
+            "end with '::patch::something'";
 
-    croak "$target is not loaded, please 'use $target' before patching"
+    croak "$pre: $target is not loaded, please 'use $target' before patching"
         unless is_loaded($target);
 
     my $target_ver = ${"$target\::VERSION"};
     defined($target_ver) && length($target_ver)
-        or croak "Target module '$target' does not have \$VERSION";
+        or croak "$pre: Target module '$target' does not have \$VERSION";
 
     my $pdata = $self->patch_data;
     ref($pdata) eq 'HASH'
@@ -64,7 +82,7 @@ sub import {
     #$log->tracef("Patch module config: %s", $config);
     #use Data::Dump; dd $config;
 
-    die "Unknown option: ".join(", ", keys %args) if %args;
+    croak "$pre: Unknown option: ".join(", ", keys %args) if %args;
 
     # check version
 
@@ -75,20 +93,14 @@ sub import {
     # check target version
 
     my $v_found;
-    my @all_v;
     my ($v0, $pvdata);
     while (($v0, $pvdata) = each %$vers) {
-        my @v = split /[,;]?\s+/, $v0;
-        push @all_v, @v;
-        if ($target_ver ~~ @v) {
-            $v_found++;
-            last;
-        }
+        do { $v_found++; last } if __match_v($target_ver, $v0);
     }
     unless ($v_found) {
-        my $msg = "Target module '$target' version not supported by patch ".
-            "module '$self', only these version(s) supported: ".
-                join(" ", @all_v);
+        my $msg = "$pre: Target module '$target' version not supported by ".
+            "patch module '$self', only these version(s) supported: ".
+                join(" ", sort keys %$vers);
         if ($on_uv eq 'ignore') {
             # do not warn, but do nothing
             return;
@@ -118,11 +130,7 @@ sub import {
         my $opvdata;
         my $c;
         while (($v0, $opvdata) = each %$overs) {
-            my @v = split /[,;]?\s+/, $v0;
-            if ($target_ver ~~ @v) {
-                $c++;
-                last;
-            }
+            do { $c++; last } if __match_v($target_ver, $v0);
         }
         if ($c) {
             my $osubs = [keys %{$opvdata->{subs}}];
@@ -135,7 +143,7 @@ sub import {
     }
 
     if (@conflicts) {
-        my $msg = "Patch module '$self' conflicts with other loaded ".
+        my $msg = "$pre: Patch module '$self' conflicts with other loaded ".
             "patch modules, here are the conflicting subroutines: ".
                 join(", ", @conflicts);
         if ($on_c eq 'ignore') {
@@ -156,10 +164,12 @@ sub import {
     # patch!
 
     while (my ($n, $sub) = each %{$pvdata->{subs}}) {
+        croak "$pre: Target subroutine $target\::$n does not exist"
+            unless defined(&{"$target\::$n"});
+
         $handle->{$n} = patch_package $target, $n, $sub;
     }
     push @{ $applied_patches{$target} }, $self;
-
 }
 
 sub unimport {
@@ -188,7 +198,7 @@ Module::Patch - Base class for patch module
 
 =head1 VERSION
 
-version 0.03
+version 0.04
 
 =head1 SYNOPSIS
 
@@ -198,21 +208,30 @@ version 0.03
  use parent qw(Module::Patch);
 
  sub patch_data {
-     my $foo = sub {
+     my $my_foo = sub {
          my $orig = shift;
          ...
      };
      return {
          versions => {
+             # version specification can be a single version string
              '1.00' => {
                  subs => {
                      foo => $my_foo,
+                     bar => sub { ... },
+                     ...
                  },
              },
-             '1.02 1.03' => {
-                 subs => {
-                     foo => $my_foo,
-                 },
+
+             # or multiple versions, separated by whitespace
+             '1.02 1.03 /^2\..+$/' => {
+                 ...
+             },
+
+             # also can contain a regex (/.../), no spaces in regex though. and
+             # watch out for escapes.
+             '1.99 /^2[.].+$/' => {
+                 ...
              },
          },
      };
